@@ -1,5 +1,5 @@
 /****************************************************************************
- * Copyright 2018-2024,2025 Thomas E. Dickey                                *
+ * Copyright 2018-2025,2026 Thomas E. Dickey                                *
  * Copyright 1998-2017,2018 Free Software Foundation, Inc.                  *
  *                                                                          *
  * Permission is hereby granted, free of charge, to any person obtaining a  *
@@ -35,7 +35,7 @@
  ****************************************************************************/
 
 /*
- * $Id: curses.priv.h,v 1.713 2025/08/16 14:40:02 tom Exp $
+ * $Id: curses.priv.h,v 1.744 2026/01/13 08:51:14 tom Exp $
  *
  *	curses.priv.h
  *
@@ -90,10 +90,8 @@ extern int errno;
 /* Some Windows related defines */
 
 #undef _NC_WINDOWS
-#if (defined(_WIN32) || defined(_WIN64__) || defined(__MSYS__) || defined(__CYGWIN__))
+#if (defined(_WIN32) || defined(_WIN64__) || defined(__MSYS__))
 #define _NC_WINDOWS
-#else
-#undef EXP_WIN32_DRIVER
 #endif
 
 #undef _NC_CYGWIN
@@ -313,7 +311,7 @@ extern NCURSES_EXPORT(void *) _nc_memmove (void *, const void *, size_t);
 /*
  * Options for terminal drivers, etc...
  */
-#ifdef USE_TERM_DRIVER
+#if USE_TERM_DRIVER
 #define NO_TERMINAL "unknown"
 #define USE_SP_RIPOFF     1
 #define USE_SP_TERMTYPE   1
@@ -420,8 +418,21 @@ typedef TRIES {
 #define TWAIT_MASK TW_ANY
 #endif
 
-#if defined(USE_WIN32CON_DRIVER)
+#include <term.priv.h>		/* defines TERMIOS via term.h */
+
+#if USE_TERM_DRIVER
+#if defined(TERMIOS)
+#undef  USE_NAMED_PIPES
+#define USE_NAMED_PIPES 0
+#undef  USE_WIN32CON_DRIVER
+#elif defined(_NC_WINDOWS)
 #include <nc_win32.h>
+#endif /* TERMIOS */
+#endif /* USE_TERM_DRIVER */
+
+#ifndef FixupPathname
+#define FixupPathname(path) /* nothing */
+#define FixupPathname2(path,buffer) /* nothing */
 #endif
 
 typedef struct
@@ -467,7 +478,6 @@ typedef union {
 
 #include <nc_panel.h>
 
-#include <term.priv.h>
 #include <nc_termios.h>
 
 #define IsPreScreen(sp)      (((sp) != NULL) && sp->_prescreen)
@@ -835,7 +845,7 @@ typedef enum {
 #if USE_SYSMOUSE
 	,M_SYSMOUSE		/* FreeBSD sysmouse on console */
 #endif
-#ifdef USE_TERM_DRIVER
+#if USE_TERM_DRIVER
 	,M_TERM_DRIVER		/* Win32 console, etc */
 #endif
 } MouseType;
@@ -941,10 +951,16 @@ typedef int (*TYPE_Gpm_GetEvent) (Gpm_Event *);
  * The filesystem database normally uses a single-letter for the lower level
  * of directories.  Use a hexadecimal code for filesystems which do not
  * preserve mixed-case names.
+ *
+ * The leading "./" is used in _nc_to_dospath() to decide whether to prefix
+ * the converted path with the MinGW32 base directory.
  */
 #if MIXEDCASE_FILENAMES
 #define LEAF_FMT "%c"
 #define LEAF_LEN 1
+#elif USE_DOS_PATHS
+#define LEAF_FMT "./%02x"
+#define LEAF_LEN 4
 #else
 #define LEAF_FMT "%02x"
 #define LEAF_LEN 2
@@ -961,7 +977,7 @@ typedef int (*TYPE_Gpm_GetEvent) (Gpm_Event *);
 #define TRACEMSE_MAX	(80 + (5 * 10) + (32 * 15))
 #define TRACEMSE_FMT	"id %2d  at (%2d, %2d, %2d) state %4lx = {" /* } */
 
-#ifdef USE_TERM_DRIVER
+#if USE_TERM_DRIVER
 struct DriverTCB; /* Terminal Control Block forward declaration */
 #define INIT_TERM_DRIVER()	_nc_globals.term_driver = _nc_get_driver
 #else
@@ -1152,7 +1168,8 @@ typedef struct screen {
 	MouseFormat	_mouse_format;	/* type of xterm mouse protocol */
 	NCURSES_CONST char *_mouse_xtermcap; /* string to enable/disable mouse */
 	MEVENT		_mouse_events[EV_MAX];	/* hold the last mouse event seen */
-	MEVENT		*_mouse_eventp;	/* next free slot in event queue */
+	MEVENT		*_mouse_readp;	/* read pointer into event queue */
+	MEVENT		*_mouse_writep;	/* write pointer into event queue */
 
 	/*
 	 * These are data that support the proper handling of the panel stack on an
@@ -1238,7 +1255,7 @@ typedef struct screen {
 	int		_sysmouse_new_buttons;
 #endif
 
-#if defined(USE_TERM_DRIVER) || defined(EXP_WIN32_DRIVER)
+#if USE_TERM_DRIVER || USE_NAMED_PIPES
 	MEVENT		_drv_mouse_fifo[FIFO_SIZE];
 	int		_drv_mouse_head;
 	int		_drv_mouse_tail;
@@ -1484,7 +1501,7 @@ extern NCURSES_EXPORT_VAR(SIG_ATOMIC_T) _nc_have_sigwinch;
 #define PUTC(ch)	do { if(!isWidecExt(ch)) {				    \
 			if (Charable(ch)) {					    \
 			    TR_PUTC(CharOf(ch));				    \
-			    NCURSES_OUTC_FUNC (NCURSES_SP_ARGx CharOf(ch));	    \
+			    NCURSES_OUTC_FUNC (NCURSES_SP_ARGx (int) CharOf(ch));	    \
 			    COUNT_OUTCHARS(1);					    \
 			} else {						    \
 			    for (PUTC_i = 0; PUTC_i < CCHARW_MAX; ++PUTC_i) {	    \
@@ -1497,15 +1514,19 @@ extern NCURSES_EXPORT_VAR(SIG_ATOMIC_T) _nc_have_sigwinch;
 				if (PUTC_n <= 0) {				    \
 				    if (PUTC_ch && is8bits(PUTC_ch) && PUTC_i == 0) { \
 					TR_PUTC(CharOf(ch));			    \
-					NCURSES_OUTC_FUNC (NCURSES_SP_ARGx CharOf(ch)); \
+					NCURSES_OUTC_FUNC (NCURSES_SP_ARGx (int) CharOf(ch)); \
 				    }						    \
 				    break;					    \
-				} else {					    \
+				} else if (PUTC_n > 1 || !is8bits(PUTC_ch)) {	    \
 				    int PUTC_j;					    \
 				    for (PUTC_j = 0; PUTC_j < PUTC_n; ++PUTC_j) {   \
 					TR_PUTC(PUTC_buf[PUTC_j]);		    \
 					NCURSES_OUTC_FUNC (NCURSES_SP_ARGx PUTC_buf[PUTC_j]); \
 				    }						    \
+				} else {					    \
+				    PUTC_buf[0] = (char) PUTC_ch;			    \
+				    TR_PUTC(PUTC_buf[0]);			    \
+				    NCURSES_OUTC_FUNC (NCURSES_SP_ARGx PUTC_buf[0]); \
 				}						    \
 			    }							    \
 			    COUNT_OUTCHARS(PUTC_i);				    \
@@ -1815,6 +1836,8 @@ extern NCURSES_EXPORT(const char *) _nc_viscbuf (const NCURSES_CH_T *, int);
 
 #define START_TRACE() /* nothing */
 
+#define USE_TRACEF(mask) 0
+
 #define T(a)
 #define TR(n, a)
 #define TPUTS_TRACE(s)
@@ -1980,7 +2003,7 @@ extern NCURSES_EXPORT(void) _nc_expanded (void);
 	}
 
 #if !NCURSES_WCWIDTH_GRAPHICS
-extern NCURSES_EXPORT(int) _nc_wacs_width(unsigned);
+extern NCURSES_EXPORT(int) _nc_wacs_width(wchar_t);
 #else
 #define _nc_wacs_width(ch) wcwidth(ch)
 #endif
@@ -2273,7 +2296,7 @@ extern NCURSES_EXPORT(int) _nc_eventlist_timeout(_nc_eventlist *);
  */
 #if USE_WIDEC_SUPPORT
 
-#if defined(_NC_WINDOWS_NATIVE) && !defined(_NC_MSC) && !defined(EXP_WIN32_DRIVER)
+#if defined(_NC_WINDOWS_NATIVE) && !defined(_NC_MSC) && !USE_NAMED_PIPES
 /*
  * MinGW has wide-character functions, but they do not work correctly.
  */
@@ -2399,7 +2422,7 @@ extern NCURSES_EXPORT(int) _nc_get_tty_mode(TTY *);
     }\
     sp->jump = outc
 
-#ifdef USE_TERM_DRIVER
+#if USE_TERM_DRIVER
 
 typedef struct _termInfo
 {
@@ -2496,7 +2519,7 @@ extern NCURSES_EXPORT(void)     _nc_get_screensize_ex(SCREEN *, TERMINAL *, int
  * Entrypoints which are actually provided in the terminal driver, which would
  * be an sp-name otherwise.
  */
-#ifdef USE_TERM_DRIVER
+#if USE_TERM_DRIVER
 #define TINFO_HAS_KEY           _nc_tinfo_has_key
 #define TINFO_DOUPDATE          _nc_tinfo_doupdate
 #define TINFO_MVCUR             _nc_tinfo_mvcur
@@ -2525,7 +2548,7 @@ extern NCURSES_EXPORT(int)      TINFO_MVCUR(SCREEN*, int, int, int, int);
 /*
  * Entrypoints using an extra parameter with the terminal driver.
  */
-#ifdef USE_TERM_DRIVER
+#if USE_TERM_DRIVER
 extern NCURSES_EXPORT(void)   _nc_get_screensize(SCREEN *, TERMINAL *, int *, int *);
 extern NCURSES_EXPORT(int)    _nc_setupterm_ex(TERMINAL **, const char *, int , int *, int);
 #define TINFO_GET_SIZE(sp, tp, lp, cp) \
@@ -2544,29 +2567,46 @@ extern NCURSES_EXPORT(void)   _nc_get_screensize(SCREEN *, int *, int *);
 	_nc_setupterm(name, fd, err, reuse)
 #endif /* !USE_TERM_DRIVER */
 
-#ifdef EXP_WIN32_DRIVER
-extern NCURSES_EXPORT_VAR(TERM_DRIVER) _nc_TINFO_DRIVER;
+#if USE_TERM_DRIVER
 extern NCURSES_EXPORT_VAR(TERM_DRIVER) _nc_WIN_DRIVER;
-#else
-#ifdef USE_TERM_DRIVER
-#if defined(USE_WIN32CON_DRIVER)
-extern NCURSES_EXPORT_VAR(TERM_DRIVER) _nc_WIN_DRIVER;
-extern NCURSES_EXPORT(int) _nc_mingw_console_read(SCREEN *sp, HANDLE fd, int *buf);
-extern NCURSES_EXPORT(int) _nc_mingw_isatty(int fd);
-extern NCURSES_EXPORT(int) _nc_mingw_isconsole(int fd);
-extern NCURSES_EXPORT(int) _nc_mingw_tcflush(int fd, int queue);
-extern NCURSES_EXPORT(int) _nc_mingw_tcgetattr(int fd, struct termios *arg);
-extern NCURSES_EXPORT(int) _nc_mingw_testmouse(SCREEN *sp, HANDLE fd, int delay EVENTLIST_2nd(_nc_eventlist*));
-#else
-#endif
 extern NCURSES_EXPORT_VAR(TERM_DRIVER) _nc_TINFO_DRIVER;
 #endif /* USE_TERM_DRIVER */
-#endif /* EXP_WIN32_DRIVER */
 
-#if defined(USE_TERM_DRIVER) && defined(EXP_WIN32_DRIVER)
+#ifdef TERMIOS
+#define USE_WINCONMODE 0
+#elif defined(USE_WIN32CON_DRIVER)
+#define USE_WINCONMODE 1
+extern NCURSES_EXPORT(int)  _nc_console_setmode(void* handle, const ConsoleMode* arg);
+extern NCURSES_EXPORT(int)  _nc_console_getmode(void* handle, ConsoleMode* arg);
+extern NCURSES_EXPORT(bool)  _nc_console_checkinit(bool assumeTermInfo);
+extern NCURSES_EXPORT(void*) _nc_console_fd2handle(int fd);
+extern NCURSES_EXPORT(WORD) _nc_console_MapColor(bool fore, int color);
+extern NCURSES_EXPORT(int)  _nc_console_flush(void* handle);
+extern NCURSES_EXPORT(bool) _nc_console_get_SBI(void);
+extern NCURSES_EXPORT(HANDLE) _nc_console_handle(int fd);
+extern NCURSES_EXPORT(int)  _nc_console_isatty(int fd);
+extern NCURSES_EXPORT(bool) _nc_console_keyExist(int keycode);
+extern NCURSES_EXPORT(int)  _nc_console_keyok(int keycode, int flag);
+extern NCURSES_EXPORT(int)  _nc_console_read(SCREEN *sp, HANDLE fd, int *buf);
+extern NCURSES_EXPORT(bool) _nc_console_restore(void);
+extern NCURSES_EXPORT(void) _nc_console_selectActiveHandle(void);
+extern NCURSES_EXPORT(void) _nc_console_set_scrollback(bool normal, CONSOLE_SCREEN_BUFFER_INFO * info);
+extern NCURSES_EXPORT(void) _nc_console_size(int *Lines, int *Cols);
+extern NCURSES_EXPORT(int)  _nc_console_test(int fd);
+extern NCURSES_EXPORT(int)  _nc_console_testmouse(const SCREEN *sp, HANDLE fd, int delay EVENTLIST_2nd(_nc_eventlist*));
+extern NCURSES_EXPORT(int)  _nc_console_twait(const SCREEN *sp, HANDLE hdl,int mode,int msec,int *left EVENTLIST_2nd(_nc_eventlist * evl));
+extern NCURSES_EXPORT(int)  _nc_console_vt_supported(void);
+
+#ifdef _NC_CHECK_MINTTY
+extern NCURSES_EXPORT(int)    _nc_console_checkmintty(int fd, LPHANDLE pMinTTY);
+#endif
+
+#else
+#error unsupported driver configuration
+#endif /* USE_WIN32CON_DRIVER */
+
+#if USE_TERM_DRIVER && defined(USE_WIN32CON_DRIVER)
 #define NC_ISATTY(fd) (0 != _nc_console_isatty(fd))
-#elif defined(USE_TERM_DRIVER) && defined(USE_WIN32CON_DRIVER)
-#define NC_ISATTY(fd) _nc_mingw_isatty(fd)
 #else
 #define NC_ISATTY(fd) isatty(fd)
 #endif
@@ -2579,20 +2619,20 @@ extern NCURSES_EXPORT_VAR(TERM_DRIVER) _nc_TINFO_DRIVER;
 	 && (value = ttyname(fd)) != NULL \
 	 && strncmp(value, "/dev/pts/", 9))
 
-#ifdef USE_TERM_DRIVER
-#  define IsTermInfo(sp)       ((TCBOf(sp) != NULL) && ((TCBOf(sp)->drv->isTerminfo)))
+#if USE_TERM_DRIVER
+#  define IsTermInfo(sp)       ((TCBOf(sp) != NULL) && ((TCBOf(sp)->drv != NULL)) && ((TCBOf(sp)->drv->isTerminfo)))
 #  define HasTInfoTerminal(sp) ((NULL != TerminalOf(sp)) && IsTermInfo(sp))
-#  if defined(EXP_WIN32_DRIVER)
+#  if USE_NAMED_PIPES
 #    define IsTermInfoOnConsole(sp) (IsTermInfo(sp) && _nc_console_test(TerminalOf(sp)->Filedes))
 #  elif defined(USE_WIN32CON_DRIVER)
-#    define IsTermInfoOnConsole(sp) (IsTermInfo(sp) && _nc_mingw_isconsole(TerminalOf(sp)->Filedes))
+#    define IsTermInfoOnConsole(sp) (IsTermInfo(sp) && _nc_console_test(TerminalOf(sp)->Filedes))
 #  else
 #    define IsTermInfoOnConsole(sp) FALSE
 #  endif
 #else
 #  define IsTermInfo(sp)       TRUE
 #  define HasTInfoTerminal(sp) (NULL != TerminalOf(sp))
-#  if defined(EXP_WIN32_DRIVER)
+#  if USE_NAMED_PIPES
 #    define IsTermInfoOnConsole(sp) _nc_console_test(TerminalOf(sp)->Filedes)
 #  else
 #    define IsTermInfoOnConsole(sp) FALSE
